@@ -54,7 +54,7 @@
 // Copyright (C) 2022 Felix Jung <fxjung@posteo.de>
 // Copyright (C) 2022 crt <chluo@cse.cuhk.edu.hk>
 // Copyright (C) 2022 Erich E. Hoover <erich.e.hoover@gmail.com>
-// Copyright 2023 g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
+// Copyright (C) 2023 g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -64,7 +64,6 @@
 #include <config.h>
 #include <poppler-config.h>
 
-#include <array>
 #include <cctype>
 #include <clocale>
 #include <cstdio>
@@ -100,11 +99,19 @@
 #include "PDFDoc.h"
 #include "Hints.h"
 #include "UTF.h"
+#include "FlateEncoder.h"
 #include "JSInfo.h"
 #include "ImageEmbeddingUtils.h"
 
 //------------------------------------------------------------------------
 
+struct FILECloser
+{
+    void operator()(FILE *f) { fclose(f); }
+};
+
+//------------------------------------------------------------------------
+
 #define headerSearchSize                                                                                                                                                                                                                       \
     1024 // read this many bytes at beginning of
          //   file to look for '%PDF'
@@ -820,8 +827,6 @@ Hints *PDFDoc::getHints()
 int PDFDoc::savePageAs(const GooString &name, int pageNo)
 {
     FILE *f;
-    OutStream *outStr;
-    XRef *yRef, *countRef;
 
     if (file && file->modificationTimeChangedSinceOpen()) {
         return errFileChangedSinceOpen;
@@ -854,26 +859,28 @@ int PDFDoc::savePageAs(const GooString &name, int pageNo)
         error(errIO, -1, "Couldn't open file '{0:t}'", &name);
         return errOpenFile;
     }
-    outStr = new FileOutStream(f, 0);
+    // Calls fclose on f when the fileCloser is destroyed because it goes out of scope
+    const std::unique_ptr<FILE, FILECloser> fileCloser(f);
+    const std::unique_ptr<OutStream> outStr = std::make_unique<FileOutStream>(f, 0);
 
-    yRef = new XRef(getXRef()->getTrailerDict());
+    const std::unique_ptr<XRef> yRef = std::make_unique<XRef>(getXRef()->getTrailerDict());
 
     if (secHdlr != nullptr && !secHdlr->isUnencrypted()) {
         yRef->setEncryption(secHdlr->getPermissionFlags(), secHdlr->getOwnerPasswordOk(), fileKey, keyLength, secHdlr->getEncVersion(), secHdlr->getEncRevision(), encAlgorithm);
     }
-    countRef = new XRef();
+    const std::unique_ptr<XRef> countRef = std::make_unique<XRef>();
     Object *trailerObj = getXRef()->getTrailerDict();
     if (trailerObj->isDict()) {
-        markPageObjects(trailerObj->getDict(), yRef, countRef, 0, refPage->num, rootNum + 2);
+        markPageObjects(trailerObj->getDict(), yRef.get(), countRef.get(), 0, refPage->num, rootNum + 2);
     }
     yRef->add(0, 65535, 0, false);
-    writeHeader(outStr, getPDFMajorVersion(), getPDFMinorVersion());
+    writeHeader(outStr.get(), getPDFMajorVersion(), getPDFMinorVersion());
 
     // get and mark info dict
     Object infoObj = getXRef()->getDocInfo();
     if (infoObj.isDict()) {
         Dict *infoDict = infoObj.getDict();
-        markPageObjects(infoDict, yRef, countRef, 0, refPage->num, rootNum + 2);
+        markPageObjects(infoDict, yRef.get(), countRef.get(), 0, refPage->num, rootNum + 2);
         if (trailerObj->isDict()) {
             Dict *trailerDict = trailerObj->getDict();
             const Object &ref = trailerDict->lookupNF("Info");
@@ -889,48 +896,48 @@ int PDFDoc::savePageAs(const GooString &name, int pageNo)
     // get and mark output intents etc.
     Object catObj = getXRef()->getCatalog();
     if (!catObj.isDict()) {
-        fclose(f);
-        delete yRef;
-        delete countRef;
-        delete outStr;
         error(errSyntaxError, -1, "XRef's Catalog is not a dictionary");
         return errOpenFile;
     }
     Dict *catDict = catObj.getDict();
     Object pagesObj = catDict->lookup("Pages");
+    if (!pagesObj.isDict()) {
+        error(errSyntaxError, -1, "Catalog Pages is not a dictionary");
+        return errOpenFile;
+    }
     Object afObj = catDict->lookupNF("AcroForm").copy();
     if (!afObj.isNull()) {
-        markAcroForm(&afObj, yRef, countRef, 0, refPage->num, rootNum + 2);
+        markAcroForm(&afObj, yRef.get(), countRef.get(), 0, refPage->num, rootNum + 2);
     }
     Dict *pagesDict = pagesObj.getDict();
     Object resourcesObj = pagesDict->lookup("Resources");
     if (resourcesObj.isDict()) {
-        markPageObjects(resourcesObj.getDict(), yRef, countRef, 0, refPage->num, rootNum + 2);
+        markPageObjects(resourcesObj.getDict(), yRef.get(), countRef.get(), 0, refPage->num, rootNum + 2);
     }
-    if (!markPageObjects(catDict, yRef, countRef, 0, refPage->num, rootNum + 2)) {
-        fclose(f);
-        delete yRef;
-        delete countRef;
-        delete outStr;
+    if (!markPageObjects(catDict, yRef.get(), countRef.get(), 0, refPage->num, rootNum + 2)) {
         error(errSyntaxError, -1, "markPageObjects failed");
         return errDamaged;
     }
 
+    if (!page.isDict()) {
+        error(errSyntaxError, -1, "page is not a dictionary");
+        return errOpenFile;
+    }
     Dict *pageDict = page.getDict();
     if (resourcesObj.isNull() && !pageDict->hasKey("Resources")) {
         Object *resourceDictObject = getCatalog()->getPage(pageNo)->getResourceDictObject();
         if (resourceDictObject->isDict()) {
             resourcesObj = resourceDictObject->copy();
-            markPageObjects(resourcesObj.getDict(), yRef, countRef, 0, refPage->num, rootNum + 2);
+            markPageObjects(resourcesObj.getDict(), yRef.get(), countRef.get(), 0, refPage->num, rootNum + 2);
         }
     }
-    markPageObjects(pageDict, yRef, countRef, 0, refPage->num, rootNum + 2);
+    markPageObjects(pageDict, yRef.get(), countRef.get(), 0, refPage->num, rootNum + 2);
     Object annotsObj = pageDict->lookupNF("Annots").copy();
     if (!annotsObj.isNull()) {
-        markAnnotations(&annotsObj, yRef, countRef, 0, refPage->num, rootNum + 2);
+        markAnnotations(&annotsObj, yRef.get(), countRef.get(), 0, refPage->num, rootNum + 2);
     }
     yRef->markUnencrypted();
-    writePageObjects(outStr, yRef, 0);
+    writePageObjects(outStr.get(), yRef.get(), 0);
 
     yRef->add(rootNum, 0, outStr->getPos(), true);
     outStr->printf("%d 0 obj\n", rootNum);
@@ -943,7 +950,7 @@ int PDFDoc::savePageAs(const GooString &name, int pageNo)
             }
             Object value = catDict->getValNF(j).copy();
             outStr->printf("/%s ", key);
-            writeObject(&value, outStr, getXRef(), 0, nullptr, cryptRC4, 0, 0, 0);
+            writeObject(&value, outStr.get(), getXRef(), 0, nullptr, cryptRC4, 0, 0, 0);
         }
     }
     outStr->printf(">>\nendobj\n");
@@ -953,7 +960,7 @@ int PDFDoc::savePageAs(const GooString &name, int pageNo)
     outStr->printf("<< /Type /Pages /Kids [ %d 0 R ] /Count 1 ", rootNum + 2);
     if (resourcesObj.isDict()) {
         outStr->printf("/Resources ");
-        writeObject(&resourcesObj, outStr, getXRef(), 0, nullptr, cryptRC4, 0, 0, 0);
+        writeObject(&resourcesObj, outStr.get(), getXRef(), 0, nullptr, cryptRC4, 0, 0, 0);
     }
     outStr->printf(">>\n");
     outStr->printf("endobj\n");
@@ -971,7 +978,7 @@ int PDFDoc::savePageAs(const GooString &name, int pageNo)
             outStr->printf("/Parent %d 0 R", rootNum + 1);
         } else {
             outStr->printf("/%s ", key);
-            writeObject(&value, outStr, getXRef(), 0, nullptr, cryptRC4, 0, 0, 0);
+            writeObject(&value, outStr.get(), getXRef(), 0, nullptr, cryptRC4, 0, 0, 0);
         }
     }
     outStr->printf(" >>\nendobj\n");
@@ -981,13 +988,9 @@ int PDFDoc::savePageAs(const GooString &name, int pageNo)
     ref.num = rootNum;
     ref.gen = 0;
     Object trailerDict = createTrailerDict(rootNum + 3, false, 0, &ref, getXRef(), name.c_str(), uxrefOffset);
-    writeXRefTableTrailer(std::move(trailerDict), yRef, false /* do not write unnecessary entries */, uxrefOffset, outStr, getXRef());
+    writeXRefTableTrailer(std::move(trailerDict), yRef.get(), false /* do not write unnecessary entries */, uxrefOffset, outStr.get(), getXRef());
 
     outStr->close();
-    fclose(f);
-    delete yRef;
-    delete countRef;
-    delete outStr;
 
     return errNone;
 }
@@ -1401,8 +1404,8 @@ void PDFDoc::writeObject(Object *obj, OutStream *outStr, XRef *xRef, unsigned in
             // we write the stream unencoded => TODO: write stream encoder
 
             // Encrypt stream
-            EncryptStream *encStream = nullptr;
             bool removeFilter = true;
+            bool addEncryptstream = false;
             if (stream->getKind() == strWeird && fileKey) {
                 Object filter = stream->getDict()->lookup("Filter");
                 if (!filter.isName("Crypt")) {
@@ -1415,22 +1418,30 @@ void PDFDoc::writeObject(Object *obj, OutStream *outStr, XRef *xRef, unsigned in
                             }
                         }
                         if (removeFilter) {
-                            encStream = new EncryptStream(stream, fileKey, encAlgorithm, keyLength, ref);
-                            encStream->setAutoDelete(false);
-                            stream = encStream;
+                            addEncryptstream = true;
                         }
                     } else {
-                        encStream = new EncryptStream(stream, fileKey, encAlgorithm, keyLength, ref);
-                        encStream->setAutoDelete(false);
-                        stream = encStream;
+                        addEncryptstream = true;
                     }
                 } else {
                     removeFilter = false;
                 }
             } else if (fileKey != nullptr) { // Encrypt stream
-                encStream = new EncryptStream(stream, fileKey, encAlgorithm, keyLength, ref);
+                addEncryptstream = true;
+            }
+
+            std::unique_ptr<EncryptStream> encStream;
+            std::unique_ptr<Stream> compressStream;
+            Object filter = stream->getDict()->lookup("Filter");
+            if (filter.isName("FlateDecode")) {
+                compressStream = std::make_unique<FlateEncoder>(stream);
+                stream = compressStream.get();
+                removeFilter = false;
+            }
+            if (addEncryptstream) {
+                encStream = std::make_unique<EncryptStream>(stream, fileKey, encAlgorithm, keyLength, ref);
                 encStream->setAutoDelete(false);
-                stream = encStream;
+                stream = encStream.get();
             }
 
             stream->reset();
@@ -1453,7 +1464,6 @@ void PDFDoc::writeObject(Object *obj, OutStream *outStr, XRef *xRef, unsigned in
 
             writeDictionary(stream->getDict(), outStr, xRef, numOffset, fileKey, encAlgorithm, keyLength, ref, alreadyWrittenDicts);
             writeStream(stream, outStr);
-            delete encStream;
         } else if (fileKey != nullptr && stream->getKind() == strFile && static_cast<FileStream *>(stream)->getNeedsEncryptionOnSave()) {
             EncryptStream *encStream = new EncryptStream(stream, fileKey, encAlgorithm, keyLength, ref);
             encStream->setAutoDelete(false);
@@ -2154,24 +2164,6 @@ bool PDFDoc::hasJavascript()
     return jsInfo.containsJS();
 }
 
-static std::string findPdfFontNameToUseForSigning(Form *form)
-{
-    static constexpr std::array<const char *, 2> fontsToUseToSign = { "Helvetica", "Arial" };
-    for (const char *fontToUseToSign : fontsToUseToSign) {
-        std::string pdfFontName = form->findFontInDefaultResources(fontToUseToSign, "");
-        if (!pdfFontName.empty()) {
-            return pdfFontName;
-        }
-
-        pdfFontName = form->addFontToDefaultResources(fontToUseToSign, "").fontName;
-        if (!pdfFontName.empty()) {
-            return pdfFontName;
-        }
-    }
-
-    return {};
-}
-
 bool PDFDoc::sign(const std::string &saveFilename, const std::string &certNickname, const std::string &password, GooString *partialFieldName, int page, const PDFRectangle &rect, const GooString &signatureText,
                   const GooString &signatureTextLeft, double fontSize, double leftFontSize, std::unique_ptr<AnnotColor> &&fontColor, double borderWidth, std::unique_ptr<AnnotColor> &&borderColor,
                   std::unique_ptr<AnnotColor> &&backgroundColor, const GooString *reason, const GooString *location, const std::string &imagePath, const std::optional<GooString> &ownerPassword, const std::optional<GooString> &userPassword)
@@ -2189,7 +2181,7 @@ bool PDFDoc::sign(const std::string &saveFilename, const std::string &certNickna
     }
 
     Form *form = catalog->getCreateForm();
-    const std::string pdfFontName = findPdfFontNameToUseForSigning(form);
+    const std::string pdfFontName = form->findPdfFontNameToUseForSigning();
     if (pdfFontName.empty()) {
         return false;
     }
